using System;
using System.Collections.Generic;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Text;
using System.Windows.Forms;
using System.Diagnostics;
using FSUIPC;//FSUIPC命名空间

namespace FSUIPCClientExample_CSharp
{
    public partial class Form1 : Form
    {
        // 私有静态成员
        private static readonly string AppTitle = "FSUIPCClientApplication_CSharp";

        // 创建这个应用中我们感兴趣的Offset
		// 下方注释为每个Offset的作用
        private Offset<int> airspeed = new Offset<int>(0x02BC);  // 基本整型读取
        private Offset<int> avionics = new Offset<int>(0x2E80);  // 基本整型读写
        private Offset<byte[]> fsLocalDateTime = new Offset<byte[]>(0x0238, 10); // 读取任意字节集合
        private Offset<string> aircraftType = new Offset<string>("AircraftInfo", 0x3160, 24); // 读取字符串，分组
        private Offset<BitArray> lights = new Offset<BitArray>(0x0D0C, 2); // 使用BitArray管理位域类型Offset
        private Offset<Double> compass = new Offset<double>(0x02CC); // 演示断开/重连
        private Offset<short> pause = new Offset<short>(0x0262, true); // 只写Offset
        private Offset<short> com2bcd = new Offset<short>(0x3118); // 读取二-十进制类型的频率
        private Offset<long> playerLatitude = new Offset<long>(0x0560); // 经纬度转换
        private Offset<long> playerLongitude = new Offset<long>(0x0568); // 经纬度转换
        private Offset<short> onGround = new Offset<short>(0x0366); // 经纬度转换
        private Offset<short> magVar = new Offset<short>(0x02A0); // 经纬度转换
        private Offset<uint> playerHeadingTrue = new Offset<uint>(0x0580); // 演示移动飞机
        private Offset<long> playerAltitude = new Offset<long>(0x0570); // 演示移动飞机
        private Offset<short> slewMode = new Offset<short>(0x05DC, true); // 演示移动飞机
        private Offset<int> sendControl = new Offset<int>(0x3110, true); // 演示移动飞机

        private readonly int REFRESH_SCENERY = 65562; // 刷新场景（refresh scenery）的数

        private FsLatLonPoint EGLL; // 声明点：EGLL
        private FsLatLonQuadrilateral runwayQuad; // 定义EGLL的27L跑道范围
        private AITrafficServices AI; // AI交通服务对象的引用

        public Form1()
        {
            InitializeComponent();
            // 设置EGLL的示例数据
            // 1. 位置
            //    使用Degrees/Minutes/Seconds构造参数实例化 FsLongitude 和 FsLatitude 类
            //    Timer1_Tick() 方法展示了一个不同的参数（原始FSUIPC数据值）
            FsLatitude lat = new FsLatitude(51, 28, 39.0d); 
            FsLongitude lon = new FsLongitude(0, -27, -41.0d); 
            EGLL = new FsLatLonPoint(lat, lon);
            // 现在定义27L跑道的四边形区域
            // 如果我们知道四个角的经纬度，定义它
            // 在这个例子中我们使用辅助函数计算点
			// 这些数据你可以在Pete Dowson的
			// MakeRunways程序的输出文件中找到
            FsLatitude rwyThresholdLat = new FsLatitude(51.464943d);
            FsLongitude rwyThresholdLon = new FsLongitude(-0.434046d);
            double rwyMagHeading = 272.7d;
            double rwyMagVariation = -3d;
            double rwyLength = 11978d;
            double rwyWidth = 164d;
			// 调用FsLatLonQuarangle静态辅助类来生成跑道的四边形
            FsLatLonPoint thresholdCentre = new FsLatLonPoint(rwyThresholdLat, rwyThresholdLon);
            double trueHeading = rwyMagHeading + rwyMagVariation;
            runwayQuad = FsLatLonQuadrilateral.ForRunway(thresholdCentre, trueHeading, rwyWidth, rwyLength);
			// 设置距离单位和AI雷达范围的默认值
            this.cbxDistanceUnits.Text = "Nautical Miles";
            this.cbxRadarRange.Text = "50";
        }

        // 应用启动，尝试打开FSUIPC连接
        private void Form1_Load(object sender, EventArgs e)
        {
            openFSUIPC();
        }

        // User pressed connect button so try again...
		// 用户按了连接按钮，再次尝试
        private void btnStart_Click(object sender, EventArgs e)
        {
            openFSUIPC();
        }

		// 打开FSUIPC，如果正常，启动计时器来循环主程序
		// 如果不正常，显示错误信息
        private void openFSUIPC()
        {
            try
            {
				// 尝试打开一个FSUIPC连接
                FSUIPCConnection.Open();
                // 成功打开，禁用连接按钮
                this.btnStart.Enabled = false;
                this.chkEnableAIRadar.Enabled = true;
                // 启动计时器
                this.timer1.Interval = 200;
                this.timer1.Enabled = true;
                // 设置AI对象
                AI = FSUIPCConnection.AITrafficServices;
            }
            catch (Exception ex)
            {
				//发生错误，显示错误信息
                MessageBox.Show(ex.Message, AppTitle, MessageBoxButtons.OK, MessageBoxIcon.Error);
                FSUIPCConnection.Close();
                Debug.WriteLine(ex.Message);
            }
        }

		//应用卸载，调用Close()清空FSUIPC使用的内存
        private void Form1_FormClosed(object sender, FormClosedEventArgs e)
        {
            FSUIPCConnection.Close();
        }

		//计时器处理窗口中的实时更新
		//默认组（即未指定组）被处理
		//每个默认组的offset都被更新
        private void timer1_Tick(object sender, EventArgs e)
        {
            //处理默认组
            try
            {
                FSUIPCConnection.Process();


				//指示空速—简单的整型返回，因此除以FSUIPC文档中给定的值、显示结果
                double airpeedKnots = ((double)airspeed.Value / 128d);
                this.txtIAS.Text = airpeedKnots.ToString("f1");


                // 航电电门
                this.chkAvionics.Checked = (avionics.Value > 0);  // 0 = Off, 1 = On.


                // 高级概念：读取原始数据块
                // FS本地日期时间
				//演示了从一个offset中取得不限长度的字节
				//我们从0x0328，包含了本地日期时间的offset中取得10字节
				//它以一个字节数组返回，我们需要自己处理所有事情
				// 1.年（从第8字节开始），2字节（Int16）
                //    使用BitConverter类转换为Int16
                short year = BitConverter.ToInt16(fsLocalDateTime.Value, 8);
                //    如果你知道怎么做，你也可以手动操作：
                //    short year = (short)(fsLocalDateTime.Value[8] + (fsLocalDateTime.Value[9] * 0x100));
                // 2. 用日期为01/01来新建一个DateTime
				//	  时间：在第0，1和2字节中（时分秒）
                DateTime fsTime = new DateTime(year, 1, 1, fsLocalDateTime.Value[0], fsLocalDateTime.Value[1], fsLocalDateTime.Value[2]);
                // 3. 取得年中的天
                //    加上我们创建的1月1日来得到最终日期
                short dayNo = BitConverter.ToInt16(fsLocalDateTime.Value, 6);
                fsTime = fsTime.Add(new TimeSpan(dayNo - 1, 0, 0, 0));
                // 打印
                this.txtFSDateTime.Text = fsTime.ToString("dddd, MMMM dd yyyy hh:mm:ss");


                //灯光
                //演示使用BitArray处理位域类型的offset
				//这些灯光是一个起始于0D0C的2字节的位域
				//我创建了一个LightType枚举（在这个文件的底部）
				//这样使得代码更清晰和易于书写
				//你当然也可以直接使用0~9的值
				//在前三种灯光里，我用注释写了另一种写法
				//即使用索引器代替枚举
				//使用BitArray中有关的比特来更新每个复选框
                this.chkBeacon.Checked = lights.Value[(int)LightType.Beacon];
                //this.chkBeacon.Checked = lights.Value[1];
                this.chkCabin.Checked = lights.Value[(int)LightType.Cabin];
                //this.chkCabin.Checked = lights.Value[9];
                this.chkInstuments.Checked = lights.Value[(int)LightType.Instruments];
                //this.chkInstuments.Checked = lights.Value[5];
                this.chkLanding.Checked = lights.Value[(int)LightType.Landing];
                this.chkLogo.Checked = lights.Value[(int)LightType.Logo];
                this.chkNavigation.Checked = lights.Value[(int)LightType.Navigation];
                this.chkRecognition.Checked = lights.Value[(int)LightType.Recognition];
                this.chkStrobes.Checked = lights.Value[(int)LightType.Strobes];
                this.chkTaxi.Checked = lights.Value[(int)LightType.Taxi];
                this.chkWing.Checked = lights.Value[(int)LightType.Wing];

                //罗盘航向
				//演示断开/重连一个offset
				//我们打印的信息忽略它的更新情况
                this.txtCompass.Text = compass.Value.ToString("F2");

                // COM2频率
				//演示将二-十进制解码为字符串
                // a. 转换为十六进制字符串
                string com2String = com2bcd.Value.ToString("X");
                // b. 加入'1'，插入小数点
                com2String = "1" + com2String.Substring(0, 2) + "." + com2String.Substring(2, 2);
                this.txtCOM2.Text = com2String;

                // 经纬度
                // 演示FsLongitude和FsLatitude如何简化经纬度操作
				// 使用FSUIPC的8字节原始数据创建两个类的新实例
                FsLongitude lon = new FsLongitude(playerLongitude.Value);
                FsLatitude lat = new FsLatitude(playerLatitude.Value);
                // 使用ToString()方法转换为人类可读字符串
				// 注意很多其他的属性从不同的数值格式中得到经纬度
                this.txtLatitude.Text = lat.ToString();
                this.txtLongitude.Text = lon.ToString();

				//使用FsLonLatPoint来计算两点间的距离和方位
				//先得到飞机当前的坐标
                FsLatLonPoint currentPosition = new FsLatLonPoint(lat, lon);
                //计算当前位置和EGLL的距离
                double distance = 0;
                switch (this.cbxDistanceUnits.Text)
                {
                    case "Nautical Miles":
                        distance = currentPosition.DistanceFromInNauticalMiles(EGLL);
                        break;
                    case "Statute Miles":
                        distance = currentPosition.DistanceFromInFeet(EGLL) / 5280d;
                        break;
                    case "Kilometres":
                        distance = currentPosition.DistanceFromInMetres(EGLL) / 1000d;
                        break;
                }
				//将距离写入文本框
                this.txtDistance.Text = distance.ToString("N2");
                //取得（真）航向
                double bearing = currentPosition.BearingTo(EGLL);
                //取得磁差
                double variation = (double)magVar.Value * 360d / 65536d;
                //减去差，得到磁航向
                bearing -= variation;
				//显示航向，加上“度”（°）符号
                this.txtBearing.Text = bearing.ToString("F0") + "\u00B0";

                //判断玩家是否在跑道上
				//检查飞机是否在地面，检查当前位置是否在
				//我们使用构造器计算出的跑道四边形内
                chkPlaneOnRunway.Checked = (this.onGround.Value == 1 && runwayQuad.ContainsPoint(currentPosition));
            }
            catch (FSUIPCException ex)
            {
                if (ex.FSUIPCErrorCode == FSUIPCError.FSUIPC_ERR_SENDMSG)
                {
					//发送错误消息 - FSUIPC连接丢失
					//显示消息，停止主计时器
					//重新点亮连接按钮
					//关闭损坏的连接
                    this.timer1.Enabled = false;
                    this.btnStart.Enabled = true;
                    this.chkEnableAIRadar.Enabled = false;
                    this.chkEnableAIRadar.Checked = false;
                    this.AIRadarTimer.Enabled = false;
                    FSUIPCConnection.Close();
                    MessageBox.Show("The connection to Flight Sim has been lost.", AppTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                }
                else
                {
                    //不是连接丢失，是其他错误
                    //重抛异常
                    throw ex;
                }
            }
            catch (Exception)
            {
				//连接丢失时，有时脏数据会返回并在程序其他位置引起问题
				//这个catch块是为了不使用户看到除了FSUIPCException外的其他异常
            }
        }

        private void CheckPlayerIsOnRunway()
        {
            FsLongitude lon = new FsLongitude(playerLongitude.Value);
            FsLatitude lat = new FsLatitude(playerLatitude.Value);
            //得到飞机当前位置点
            FsLatLonPoint currentPosition = new FsLatLonPoint(lat, lon);

            // 现在定义27L跑道的四边形区域
            // 如果我们知道四个角的经纬度，定义它
            // 在这个例子中我们使用辅助函数计算点
			// 这些数据你可以在Pete Dowson的
			// MakeRunways程序的输出文件中找到
            FsLatitude rwyThresholdLat = new FsLatitude(51.464943d);
            FsLongitude rwyThresholdLon = new FsLongitude(-0.434046d);
            double rwyMagHeading = 272.7d;
            double rwyMagVariation = -3d;
            double rwyLength = 11978d;
            double rwyWidth = 164d;
            
			//调用FsLatLonQuarangle类的静态辅助函数来生成这条跑道的四边形
            FsLatLonPoint thresholdCentre = new FsLatLonPoint(rwyThresholdLat, rwyThresholdLon);
            double trueHeading = rwyMagHeading + rwyMagVariation;
            runwayQuad = FsLatLonQuadrilateral.ForRunway(thresholdCentre, trueHeading, rwyWidth, rwyLength);
            
			//现在检查玩家是否在跑道上：
			//检查飞机是否在地面，检查当前位置是否在
			//我们使用构造器计算出的跑道四边形内
            if (this.onGround.Value == 1 && runwayQuad.ContainsPoint(currentPosition))
            {
                //玩家在跑道上
				//做你需要做的事
            }
        }

		//演示分组功能
		//AircraftType（飞机种类）offset在一个名为AircraftInf的组内
        {
            // Aircraft type is in the "AircraftInfo" data group so we only want to proccess that here.
			//飞机种类在AircraftInfo组内，我们在这里只执行这个
            try
            {
                FSUIPCConnection.Process("AircraftInfo");
				//成功，显示字符串
				//DLL自动处理字符集转换
				//与处理'\0'字符串结束符
                this.txtAircraftType.Text = aircraftType.Value;
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, AppTitle, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }


        // Next few eventhandlers deal with writing the lights.
		//下面的这些事件，处理写灯光
		//我们再次使用BitArray来管理独立的比特
		//我也再次使用了LightType枚举，你可以不必如此
		//前两个方法中注释行提供了使用索引器的替代方法
        private void chkNavigation_CheckedChanged(object sender, EventArgs e)
        {
            this.lights.Value[(int)LightType.Navigation] = this.chkNavigation.Checked;
            //this.lights.Value[0] = this.chkNavigation.Checked;
        }

        private void chkBeacon_CheckedChanged(object sender, EventArgs e)
        {
            this.lights.Value[(int)LightType.Beacon] = this.chkBeacon.Checked;
            //this.lights.Value[1] = this.chkBeacon.Checked;
        }

        private void chkLanding_CheckedChanged(object sender, EventArgs e)
        {
            this.lights.Value[(int)LightType.Landing] = this.chkLanding.Checked;
        }

        private void chkTaxi_CheckedChanged(object sender, EventArgs e)
        {
            this.lights.Value[(int)LightType.Taxi] = this.chkTaxi.Checked;
        }

        private void chkStrobes_CheckedChanged(object sender, EventArgs e)
        {
            this.lights.Value[(int)LightType.Strobes] = this.chkStrobes.Checked;
        }

        private void chkInstuments_CheckedChanged(object sender, EventArgs e)
        {
            this.lights.Value[(int)LightType.Instruments] = this.chkInstuments.Checked;
        }

        private void chkRecognition_CheckedChanged(object sender, EventArgs e)
        {
            this.lights.Value[(int)LightType.Recognition] = this.chkRecognition.Checked;
        }

        private void chkWing_CheckedChanged(object sender, EventArgs e)
        {
            this.lights.Value[(int)LightType.Wing] = this.chkWing.Checked;
        }

        private void chkLogo_CheckedChanged(object sender, EventArgs e)
        {
            this.lights.Value[(int)LightType.Logo] = this.chkLogo.Checked;
        }

        private void chkCabin_CheckedChanged(object sender, EventArgs e)
        {
            this.lights.Value[(int)LightType.Cabin] = this.chkCabin.Checked;
        }

		//演示简单的写入
		//改变Value属性以向FSUIPC写入
		//在下一次Process()时，新数据将被写入
        private void chkAvionics_CheckedChanged(object sender, EventArgs e)
        {
            this.avionics.Value = chkAvionics.Checked ? 1 : 0;
        }
		
		//这里演示了断开一个独立的offset的方法
		//断开后，它不再从FSUIPC获取更新
		//也不再写入新值
        private void btnDisconnect_Click(object sender, EventArgs e)
        {
            //立刻断开
            this.compass.Disconnect();
        }

		//和断开一样，只是在下一次Process后断开
		//所以这个offset还可以进行一次读写
        private void btnDisconnectAfterNext_Click(object sender, EventArgs e)
        {
            //下次Process后断开
            this.compass.Disconnect(true);
        }

		//这里演示重连一个offset
		//它在随后的Process()调用被读写
        private void btnReconnect_Click(object sender, EventArgs e)
        {
            //重连
            this.compass.Reconnect();
        }

		//和重连一样，只是在下一次Process后重连然后又断开
        private void btnReconnectOnce_Click(object sender, EventArgs e)
        {
            //重连，但是只对一次Process()有效  
            //然后这个offset又断开了
            this.compass.Reconnect(true);
        }

		//暂停offset是只写的，它的值永远不会被从FSUIPC读取
		//当它的值改变时，将在下次Process()中写入
        private void chkPause_CheckedChanged(object sender, EventArgs e)
        {
            pause.Value = (short)(this.chkPause.Checked ? 1 : 0);
        }

        private enum LightType
        {
            Navigation,
            Beacon,
            Landing,
            Taxi,
            Strobes,
            Instruments,
            Recognition,
            Wing,
            Logo,
            Cabin
        }

        private void chkEnableAIRadar_CheckedChanged(object sender, EventArgs e)
        {
			//打开/关闭雷达计时器来更新雷达信息（每秒一次）
            this.AIRadarTimer.Enabled = this.chkEnableAIRadar.Checked;
			//强制Panel（面板）每秒重画
            this.pnlAIRadar.Invalidate();
        }

        private void AIRadarTimer_Tick(object sender, EventArgs e)
        {
			//每秒我们更新地面和空中的AI交通信息
            AI.RefreshAITrafficInformation(this.chkShowGroundAI.Checked, this.chkShowAirborneAI.Checked);
			//使用用户设置的范围激活过滤器
			//过滤地面和空中的交通，没有方位过滤（包括0~360度）
			//没有高度过滤器
			//范围在复选框被设置
            AI.ApplyFilter(true, true, 0, 360, null, null, double.Parse(this.cbxRadarRange.Text));
            //Invalidate()使其重画
            this.pnlAIRadar.Invalidate();
        }

        private void pnlAIRadar_Paint(object sender, PaintEventArgs e)
        {
            //重画时需要调用
            if (this.chkEnableAIRadar.Checked)
            {
                //清空面板，绘制黑色底色
                e.Graphics.Clear(Color.Black);
                //画中央的十字
                Point centre = new Point(this.pnlAIRadar.ClientSize.Width / 2, this.pnlAIRadar.ClientSize.Height / 2);
                e.Graphics.DrawLine(Pens.White, centre.X - 4, centre.Y, centre.X + 4, centre.Y);
                e.Graphics.DrawLine(Pens.White, centre.X, centre.Y - 4, centre.X, centre.Y + 4);
                double range = double.Parse(this.cbxRadarRange.Text);
                //处理比例
                double scale = range / (double)(this.pnlAIRadar.ClientSize.Width < this.pnlAIRadar.ClientSize.Height ? this.pnlAIRadar.ClientSize.Width : this.pnlAIRadar.ClientSize.Height) * 2d;
				//将每架飞机绘制在雷达上
				//注意：我们将地面和空中的飞机分离在两个集合中
				//这里有一个集合叫做AllTraffic，你如果不想分离，用这个
				//如果需要绘制地面AI，绘制它们
                if (this.chkShowGroundAI.Checked)
                {
                    //循环地面AI集合
                    foreach (AIPlaneInfo plane in AI.GroundTraffic)
                    {
						//这里我们传递飞机信息到绘制程序
						//AIPlaneInfo对象有不少信息
						//参考手册或Intellisense会告诉你详细信息
                        drawTarget(e.Graphics, scale, centre, plane);
                    }
                }
                //如果需要绘制空中AI，绘制它们
                if (this.chkShowAirborneAI.Checked)
                {
                    //循环空中AI集合
                    foreach (AIPlaneInfo plane in AI.AirbourneTraffic)
                    {
                        drawTarget(e.Graphics, scale, centre, plane);
                    }
                }
            }
            else
            {
                //雷达关闭，面板清空
                e.Graphics.Clear(Color.White);
            }
        }

        private void drawTarget(Graphics graphics, double scale, Point centre, AIPlaneInfo plane)
        {
			//我们将从飞机对象中提取某些有用信息来绘制目标
			//其他应用可以用到更多的其他信息
			//看参考手册或Intellisense来取得更多详情
			//将距离除以比例来得到像素
            double distancePixels = plane.DistanceNM / scale;
			//使用距离和方位来找到与中心的相对位置
            double dx = Math.Cos(degreeToRadian(plane.BearingTo)) * distancePixels;
            double dy = Math.Sin(degreeToRadian(plane.BearingTo)) * distancePixels;
            PointF target = new PointF((float)centre.X + (float)dx, (float)centre.Y + (float)dy);
			//绘制面向飞机航向的目标圆圈
            graphics.DrawEllipse(Pens.LightGreen, target.X - 4f, target.Y - 4f, 8f, 8f);
            //从圆圈绘制一条航向指示线
            double tailHeading = 180d + plane.HeadingDegrees;
            dx = Math.Cos(degreeToRadian(tailHeading)) * 12;
            dy = Math.Sin(degreeToRadian(tailHeading)) * 12;
            PointF tailEnd = new PointF(target.X + (float)dx, target.Y + (float)dy);
            graphics.DrawLine(new Pen(new LinearGradientBrush(target, tailEnd, Color.LightGreen, Color.DarkGreen)), target, tailEnd);
			//处理数据块的位置
            PointF dataBlock = new PointF(target.X + 20, target.Y - 20);
			//画线
            graphics.DrawLine(Pens.LightGreen, new PointF(target.X + 5, target.Y - 5), new PointF(dataBlock.X - 5, dataBlock.Y + 7));
			//画数据块
			//第一行：呼号
            graphics.DrawString(plane.ATCIdentifier, this.pnlAIRadar.Font, Brushes.LightGreen, dataBlock);
			//第二行：高度（百英尺）和速度
            string line2 = "";
            line2 += ((int)(plane.AltitudeFeet / 100d)).ToString("d3");
			//通过加上+，-或=指示飞机在爬升、下降或平飞
            if (plane.VirticalSpeedFeet < 0)
            {
                line2 += "-";
            }
            else if (plane.VirticalSpeedFeet > 0)
            {
                line2 += "+";
            }
            else
            {
                line2 += "=";
            }
            graphics.DrawString(line2, this.pnlAIRadar.Font, Brushes.LightGreen, new PointF(dataBlock.X, dataBlock.Y + 12));
			//第三行：起降机场和给定跑道
            graphics.DrawString(plane.DepartureICAO + "->" + plane.DestinationICAO + " " + plane.RunwayAssigned.ToString(), this.pnlAIRadar.Font, Brushes.LightGreen, new PointF(dataBlock.X, dataBlock.Y + 24));
        }

        private double degreeToRadian(double angle)
        {
            return Math.PI * angle / 180.0;
        }

        private void btnMoveToEGLL_Click(object sender, EventArgs e)
        {
            //挂起计时器
            if (this.timer1.Enabled)
            {
                this.timer1.Enabled = false;
                if (this.chkEnableAIRadar.Checked)
                {
                    this.AIRadarTimer.Enabled = false;
                }
                //将FS设为位移模式
                slewMode.Value = 1;
                FSUIPCConnection.Process();
				//生成一个代表27L入口中心的点
                FsLatitude lat = new FsLatitude(51.464943d);
                FsLongitude lon = new FsLongitude(-0.434046d);
                FsLatLonPoint newPos = new FsLatLonPoint(lat, lon);
				//使用FsLatLonPoint类中的某个OffsetBy方法
				//将点向前移动150米
                double rwyTrueHeading =  269.7d;
                newPos = newPos.OffsetByMetres(rwyTrueHeading, 150);
				//设置新位置
                playerLatitude.Value = newPos.Latitude.ToFSUnits8();
                playerLongitude.Value = newPos.Longitude.ToFSUnits8();
				//设置航向和高度
                playerAltitude.Value = 0;
                playerHeadingTrue.Value = (uint)(rwyTrueHeading * (65536d * 65536d) / 360d);
                FSUIPCConnection.Process();
                //关闭位移模式
                slewMode.Value = 0;
                FSUIPCConnection.Process();
                //刷新scenery
                sendControl.Value = REFRESH_SCENERY;
                FSUIPCConnection.Process();
                //恢复计时器
                this.timer1.Enabled = true;
                this.AIRadarTimer.Enabled = this.chkEnableAIRadar.Checked;
            }
        }
    }

	//用于显示雷达的二次缓冲面板
	//.NET的普通面板不支持二次缓冲，也可能导致其他问题
	
    public class DoubleBufferPanel : Panel
    {
        public DoubleBufferPanel()
        {
			//将二次缓冲设置为真
            this.SetStyle(ControlStyles.DoubleBuffer |
            ControlStyles.UserPaint |
            ControlStyles.AllPaintingInWmPaint,
            true);

            this.UpdateStyles();
        }
    }
}
